﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;



public class AsteroidCreator : MonoBehaviour
{
    public GameObject smallAsteroid_ = null;
    public GameObject mediumAsteroid_ = null;
    public GameObject largeAsteroid_ = null;

    public GameObject backgroundAsteroid_ = null;

    public int backgroundAsteroidCount_ = 5;

    public int backgroundDistance_ = 200;

    // total is 1 always. rest is for small asteroids depending on level
    public float largeProbability_ = 0.1f;
    public float mediumProbability_ = 0.3f;

    public int level1AsteroidCount = 10;
    public int level2AsteroidCount = 15;
    public int level3AsteroidCount = 15;

    public float maxRotate = 1.0f;
    public float maxMove = 10.0f;
    public bool generateStart = false;

    // Start is called before the first frame update
    void Start()
    {
        if (smallAsteroid_ == null ||
            mediumAsteroid_ == null ||
            largeAsteroid_ == null )
        {
            Debug.LogError("ASTEROID NOT SET: PLEASE SET ASTEROID PREFAB");
            return;
        }
        if (generateStart) createBackgroundAsteroids(5, 100, new Vector3(0, 0, 0));

    }

    // Update is called once per frame
    void Update()
    {

    }

    public List<GameObject> createAsteroidField(int width, int height, Vector3 position, int level)
    {
        List<GameObject> asteroids = new List<GameObject>();

        int asteroidCount = 0;

        switch (level)
        {
            case 0:
                asteroidCount = level1AsteroidCount;
                break;
            case 1:
                asteroidCount = level1AsteroidCount;
                break;
            case 2:
                asteroidCount = level2AsteroidCount;
                break;
            case 3:
                asteroidCount = level3AsteroidCount;
                break;
        }

        for (int i = 0; i < asteroidCount; ++i)
        {
            int xPos = Random.Range(-height / 2, height / 2);
            int yPos = Random.Range(-width / 2, width / 2);
            Vector3 randomPosition = new Vector3(xPos, yPos, 0);

            if (level == 0 &&
                Vector3.Distance(randomPosition, position) < 2.0f)
            {
                print("Protected start position");
                continue;
            }

            float xRot = Random.Range(-maxRotate / 2, maxRotate / 2);
            float yRot = Random.Range(-maxRotate / 2, maxRotate / 2);
            float zRot = Random.Range(-maxRotate / 2, maxRotate / 2);

            float xMov = Random.Range(-maxMove / 2, maxMove / 2);
            float yMov = Random.Range(-maxMove / 2, maxMove / 2);


            float asteroidType = Random.Range(0.0f, 1.0f);

            GameObject ast = null;

            if (level >= 3 && asteroidType < largeProbability_)
            {
                ast = GameObject.Instantiate(largeAsteroid_);
            }
            else if (level >= 2 && asteroidType < largeProbability_ + mediumProbability_ && asteroidType > largeProbability_)
            {
                ast = GameObject.Instantiate(mediumAsteroid_);
            }
            else
            {
                ast = GameObject.Instantiate(smallAsteroid_);
            }

            ast.transform.position = randomPosition + position;

            //ast.GetComponent<AsteroidMovement>().rotation_ = new Vector3(xRot, yRot, zRot);
            //ast.GetComponent<AsteroidMovement>().movement_ = new Vector3(xMov, yMov, 0);

            ast.GetComponent<Rigidbody>().AddForce(new Vector3(xMov, yMov, 0));
            ast.GetComponent<Rigidbody>().AddTorque(new Vector3(xRot, yRot, zRot));

            asteroids.Add(ast);
        }

        return asteroids;
    }


    public List<GameObject> createBackgroundAsteroids(int width, int height, Vector3 position)
    {
        List<GameObject> asteroids = new List<GameObject>();

        for (int i = 0; i < backgroundAsteroidCount_; ++i)
        {
            int xPos = Random.Range(-height / 2, height / 2);
            int yPos = Random.Range(-width / 2, width / 2);
            int zPos = Random.Range(0, backgroundDistance_);
            Vector3 randomPosition = new Vector3(xPos, yPos, zPos);

            float xRot = Random.Range(0, 360);
            float yRot = Random.Range(0, 360);
            float zRot = Random.Range(0, 360);

            float xMov = Random.Range(-maxMove / 2, maxMove / 2);
            float yMov = Random.Range(-maxMove / 2, maxMove / 2);


            float asteroidType = Random.Range(0.0f, 1.0f);

            GameObject ast = GameObject.Instantiate(backgroundAsteroid_); ;

            ast.transform.position = randomPosition + position;

            //ast.GetComponent<AsteroidMovement>().rotation_ = new Vector3(xRot, yRot, zRot);
            //ast.GetComponent<AsteroidMovement>().movement_ = new Vector3(xMov, yMov, 0);

            ast.transform.rotation = Quaternion.Euler(new Vector3(xRot, yRot, zRot));

            ast.GetComponent<Rigidbody>().AddForce(new Vector3(xMov/10, yMov/10, 0));
            ast.GetComponent<Rigidbody>().AddTorque(new Vector3(xRot/10, yRot/10, zRot/10));

            asteroids.Add(ast);
        }

        return asteroids;
    }
}
