﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GeneralSpawner : MonoBehaviour
{
    public GameObject player_ = null;

    public int fieldSize_ = 100;
    public int mapSize_ = 1000;

    public int backgroundAreaSize = 5;

    public int destroydistance = 2;
    public int level2Distance = 2;
    public int level3Distance = 5;


    struct Field
    {
        public bool active;
        public bool activeBackground;
        public int level;
        public List<GameObject> objects;
        public List<GameObject> backObjects;
    }

    private List<List<Field>> fields_ = new List<List<Field>>();

    private Vector3 zeroPosition_ = new Vector3(0, 0, 0);

    private float fieldCheckInterval_ = 1.0f;
    private float checkCounter_ = 0.0f;


    // Start is called before the first frame update
    void Start()
    {
        if (player_ == null)
        {
            Debug.LogError("PLAYER NOT SET: PLEASE SET PLAYER OBJECT");
            return;
        }

        for (int i = 0; i < mapSize_; ++i)
        {
            fields_.Add(new List<Field>());
            for (int j = 0; j < mapSize_; ++j)
            {
                int fieldLevel = 3;

                if (i == mapSize_ / 2 && j == mapSize_ / 2)
                {
                    fieldLevel = 0;
                }
                else if (i < mapSize_ / 2 + level2Distance &&
                    i > mapSize_ / 2 - level2Distance &&
                    j < mapSize_ / 2 + level2Distance &&
                    j > mapSize_ / 2 - level2Distance)
                {
                    fieldLevel = 1;
                }
                else if (i < mapSize_ / 2 + level3Distance &&
                    i > mapSize_ / 2 - level3Distance &&
                    j < mapSize_ / 2 + level3Distance &&
                    j > mapSize_ / 2 - level3Distance)
                {
                    fieldLevel = 2;
                }

                fields_[fields_.Count - 1].Add(new Field() { active = false, activeBackground = false, level = fieldLevel, objects = new List<GameObject>() });
            }
        }
        if (fields_.Count > 0)
        {
            print("Create fields of size: " + fields_.Count + "x" + fields_[0].Count);
        }
        else
        {
            Debug.LogError("Didn't create fields");
        }
        zeroPosition_ = player_.transform.position;
        generateFields(zeroPosition_);
    }

    // Update is called once per frame
    void Update()
    {
        checkCounter_ += Time.deltaTime;

        if (checkCounter_ > fieldCheckInterval_)
        {
            checkCounter_ -= fieldCheckInterval_;
            generateFields(player_.transform.position);
        }
    }


    void generateFields(Vector3 position)
    {
        // find the field we are located in
        int xFieldPosition = mapToFieldPosition(position.x);
        int yFieldPosition = mapToFieldPosition(position.y);

        //print("Player at field position: " + xFieldPosition + "x" + yFieldPosition);


        if (xFieldPosition <= 0 ||
            xFieldPosition >= mapSize_ - 1 ||
            yFieldPosition <= 0 ||
            yFieldPosition >= mapSize_ - 1)
        {
            print("Player outside fields");
            return;
        }

        if (!fields_[xFieldPosition - 1][yFieldPosition - 1].active)
        {
            generateOneField(xFieldPosition - 1, yFieldPosition - 1, fields_[xFieldPosition - 1][yFieldPosition].level);
        }
        if (!fields_[xFieldPosition - 1][yFieldPosition].active)
        {
            generateOneField(xFieldPosition - 1, yFieldPosition, fields_[xFieldPosition - 1][yFieldPosition].level);
        }
        if (!fields_[xFieldPosition][yFieldPosition - 1].active)
        {
            generateOneField(xFieldPosition, yFieldPosition - 1, fields_[xFieldPosition - 1][yFieldPosition].level);
        }
        if (!fields_[xFieldPosition - 1][yFieldPosition + 1].active)
        {
            generateOneField(xFieldPosition - 1, yFieldPosition + 1, fields_[xFieldPosition - 1][yFieldPosition].level);
        }
        if (!fields_[xFieldPosition + 1][yFieldPosition - 1].active)
        {
            generateOneField(xFieldPosition + 1, yFieldPosition - 1, fields_[xFieldPosition - 1][yFieldPosition].level);
        }
        if (!fields_[xFieldPosition][yFieldPosition].active)
        {
            generateOneField(xFieldPosition, yFieldPosition, fields_[xFieldPosition - 1][yFieldPosition].level);
        }
        if (!fields_[xFieldPosition + 1][yFieldPosition].active)
        {
            generateOneField(xFieldPosition + 1, yFieldPosition, fields_[xFieldPosition - 1][yFieldPosition].level);
        }
        if (!fields_[xFieldPosition][yFieldPosition + 1].active)
        {
            generateOneField(xFieldPosition, yFieldPosition + 1, fields_[xFieldPosition - 1][yFieldPosition].level);
        }
        if (!fields_[xFieldPosition + 1][yFieldPosition + 1].active)
        {
            generateOneField(xFieldPosition + 1, yFieldPosition + 1, fields_[xFieldPosition - 1][yFieldPosition].level);
        }


        // generate backgrounds
        for (int x = xFieldPosition - backgroundAreaSize; x < xFieldPosition + backgroundAreaSize; ++x)
        {
            for (int y = yFieldPosition - backgroundAreaSize; y < yFieldPosition + backgroundAreaSize; ++y)
            {
                if (!fields_[x][y].activeBackground)
                {
                    createBackground(x, y);
                }
            }
        }



        if (xFieldPosition <= destroydistance ||
            xFieldPosition >= mapSize_ - destroydistance ||
            yFieldPosition <= destroydistance ||
            yFieldPosition >= mapSize_ - destroydistance)
        {
            print("Player too close to edge to destroy");
            return;
        }

        
        int currentX = xFieldPosition - destroydistance;
        for (int y = yFieldPosition - destroydistance; y < yFieldPosition + destroydistance; ++y)
        {
            if (fields_[currentX][y].active)
            {
                destroyField(currentX, y);
            }
        }

        currentX = xFieldPosition + destroydistance;
        for (int y = yFieldPosition - destroydistance; y < yFieldPosition + destroydistance; ++y)
        {
            if (fields_[currentX][y].active)
            {
                destroyField(currentX, y);
            }
        }

        int currentY = yFieldPosition - destroydistance;
        for (int x = xFieldPosition - destroydistance; x < xFieldPosition + destroydistance; ++x)
        {
            if (fields_[x][currentY].active)
            {
                destroyField(x, currentY);
            }
        }

        currentY = yFieldPosition + destroydistance;
        for (int x = xFieldPosition - destroydistance; x < xFieldPosition + destroydistance; ++x)
        {
            if (fields_[x][currentY].active)
            {
                destroyField(x, currentY);
            }
        }







        // destroy backgrounds
        currentX = xFieldPosition - backgroundAreaSize;
        for (int y = yFieldPosition - backgroundAreaSize; y < yFieldPosition + backgroundAreaSize; ++y)
        {
            if (fields_[currentX][y].activeBackground)
            {
                destroyBackground(currentX, y);
            }
        }
        
        currentX = xFieldPosition + backgroundAreaSize;
        for (int y = yFieldPosition - backgroundAreaSize; y < yFieldPosition + backgroundAreaSize; ++y)
        {
            if (fields_[currentX][y].activeBackground)
            {
                destroyBackground(currentX, y);
            }
        }

        currentY = yFieldPosition - backgroundAreaSize;
        for (int x = xFieldPosition - backgroundAreaSize; x < xFieldPosition + backgroundAreaSize; ++x)
        { 
            if (fields_[x][currentY].activeBackground)
            {
                destroyBackground(x, currentY);
            }
        }

        currentY = yFieldPosition + backgroundAreaSize;
        for (int x = xFieldPosition - backgroundAreaSize; x < xFieldPosition + backgroundAreaSize; ++x)
        {
            if (fields_[x][currentY].activeBackground)
            {
                destroyBackground(x, currentY);
            }
        }
    }

    void generateOneField(int xFieldPosition, int yFieldPosition, int level)
    {
        //generate field
        List<GameObject> allObjects;
        List<GameObject> ships;

        allObjects = GetComponentInChildren<AsteroidCreator>().createAsteroidField(fieldSize_, fieldSize_,
            new Vector3(fieldToMapPosition(xFieldPosition), fieldToMapPosition(yFieldPosition), 0), level);

        //print("Created " + allObjects.Count + " asteroids.");

        if (level != 0)
        {
            ships = GetComponentInChildren<ShipSpawner>().createShips(new Vector2(-fieldSize_ / 2, -fieldSize_ / 2),
                new Vector2(fieldSize_ / 2, fieldSize_ / 2),
                new Vector3(fieldToMapPosition(xFieldPosition), fieldToMapPosition(yFieldPosition), 0), level);
            allObjects.AddRange(ships);
        }
        
        //print("Created " + allObjects.Count + " objects.");
        Field f = fields_[xFieldPosition][yFieldPosition];
        f.active = true;
        f.objects = allObjects;
        fields_[xFieldPosition][yFieldPosition] = f;
        //print("Created a field with: " + fields_[xFieldPosition][yFieldPosition].objects.Count + " objects.");
    }

    void createBackground(int xFieldPosition, int yFieldPosition)
    {
        List<GameObject> background;

        background = GetComponentInChildren<AsteroidCreator>().createBackgroundAsteroids(fieldSize_, fieldSize_,
             new Vector3(fieldToMapPosition(xFieldPosition), fieldToMapPosition(yFieldPosition), 0));

        Field f = fields_[xFieldPosition][yFieldPosition];
        f.activeBackground = true;
        f.backObjects = background;
        fields_[xFieldPosition][yFieldPosition] = f;
    }

    void destroyBackground(int xFieldPosition, int yFieldPosition)
    {
        Field f = fields_[xFieldPosition][yFieldPosition];
        List<GameObject> allObjects = f.backObjects;

        if (allObjects == null)
        {
            Debug.LogError("Found an active field without objects");
            return;
        }

        for (int i = 0; i < allObjects.Count; ++i)
        {
            Destroy(allObjects[i].gameObject);
        }

        f.activeBackground = false;
        f.backObjects = new List<GameObject>();
        fields_[xFieldPosition][yFieldPosition] = f;
    }

    void destroyField(int xFieldPosition, int yFieldPosition)
    {
        //print("Destroying field: " + xFieldPosition + "x" + yFieldPosition);

        Field f = fields_[xFieldPosition][yFieldPosition];
        List<GameObject> allObjects = f.objects;

        if (allObjects == null)
        {
            Debug.LogError("Found an active field without objects");
            return;
        }

        //print("Destroying: " + allObjects.Count + " objects.");
        for (int i = 0; i < allObjects.Count; ++i)
        {
            Vector3 position = allObjects[i].gameObject.transform.position;
            int xfield = mapToFieldPosition(position.x);
            int yfield = mapToFieldPosition(position.y);
            Field destroyField = fields_[xfield][yfield];

            if (!destroyField.active || (xFieldPosition == xfield && yFieldPosition == yfield))
            {
                Destroy(allObjects[i].gameObject);
            }
            else
            {
                fields_[xfield][yfield].objects.Add(allObjects[i]);
            }
        }

        f.active = false;
        f.objects = new List<GameObject>();
        fields_[xFieldPosition][yFieldPosition] = f;
    }

    int mapToFieldPosition(float position)
    {
        // center of the fields plus position in fields
        return (int)((float)mapSize_/2 + position/ fieldSize_ + 0.5f);
    }

    float fieldToMapPosition(int fieldPosition)
    {
        return (fieldPosition - mapSize_ / 2) * fieldSize_;
    }
}
