﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class LevellingUp : MonoBehaviour
{
    [System.Serializable] public struct ResourceDemands
    {
        public int magma;
        public int metal;
        public int precious;
    };

    public ResourceDemands level2 = new ResourceDemands() { magma = 0, metal = 10, precious = 0 };
    public ResourceDemands level3 = new ResourceDemands() { magma = 0, metal = 25, precious = 5 };
    public ResourceDemands level4 = new ResourceDemands() { magma = 0, metal = 100, precious = 20 };

    public Vector3 level2Size = new Vector3(1.5f, 1.5f, 1.5f);
    public Vector3 level3Size = new Vector3(1.5f, 1.5f, 1.5f);
    public Vector3 level4Size = new Vector3(1.5f, 1.5f, 1.5f);

    public int currentLevel = 1;

    public CameraFollow camera_ = null;

    public GameObject preciousIcon_ = null;
    public GameObject preciousNum_ = null;

    public GameObject planet_ = null;

    public UiManager ui_ = null;

    private PlayerResources resources = null;
    private HealthSystem health = null;


    public float checkInterval = 1.0f;
    public float checkCounter = 0.0f;


    // Start is called before the first frame update
    void Start()
    {
        resources = GetComponent<PlayerResources>();
        health = GetComponent<HealthSystem>();
        if (resources == null || ui_ == null)
        {
            Debug.LogError("Failed to find player resources or UI in levelling up");
        }

        if (preciousIcon_ == null)
        {
            Debug.LogError("Please set preciousIcon_ to player levelling up");
        }

        if (preciousNum_ == null)
        {
            Debug.LogError("Please set preciousNum_ to player levelling up");
        }

        if (camera_ == null)
        {
            Debug.LogError("set camera in levelling up player");
        }
        ui_.setCommonMax(level2.metal);
        ui_.preciousMax_ = level2.precious;
        resources.updateResourceUI();
    }

    // Update is called once per frame
    void Update()
    {
        checkCounter += Time.deltaTime;

        if (checkCounter > checkInterval)
        {
            checkCounter -= checkInterval;

            switch (currentLevel)
            {
                case 1:
                    if (resources.consumeResources(level2.magma, level2.metal, level2.precious))
                    {
                        print("Levelling up to level 2!");
                        resources.canCollectPrecious_ = true;
                        ++currentLevel;
                        transform.localScale = level2Size;
                        health.repairToFull();
                        ui_.setCommonMax(level3.metal);
                        ui_.preciousMax_ = level3.precious;

                        resources.updateResourceUI();

                        preciousIcon_.SetActive(true);
                        preciousNum_.SetActive(true);

                        camera_.cameraOffset = new Vector3(0,0, camera_.cameraOffset.z* level2Size.z);
                    }
                    break;
                case 2:
                    if (resources.consumeResources(level3.magma, level3.metal, level3.precious))
                    {
                        print("Levelling up to level 3!");
                        ++currentLevel;
                        transform.localScale = level3Size;
                        health.repairToFull();

                        ui_.setCommonMax(level4.metal);
                        ui_.preciousMax_ = level4.precious;
                        resources.updateResourceUI();

                        camera_.cameraOffset = new Vector3(0, 0, camera_.cameraOffset.z * level2Size.z);

                        
                    }
                    break;
                case 3:
                    if (resources.consumeResources(level4.magma, level4.metal, level4.precious))
                    {
                        print("Levelling up to level 4!");
                        ++currentLevel;
                        transform.localScale = level4Size;
                        health.repairToFull();
                        health.indestructable = true;

                        ui_.setCommonMax(0);
                        ui_.preciousMax_ = 0;
                        resources.updateResourceUI();

                        camera_.cameraOffset = new Vector3(0, 0, camera_.cameraOffset.z * level2Size.z);

                        GameObject.Instantiate(planet_);

                        planet_.transform.position = new Vector3(transform.position.x + 15, transform.position.y, 0);
                    }
                    break;
            }
        }
    }
}
